@@ -23,18 +23,36 @@ reticulate::use_virtualenv
 #' @export
 reticulate::use_condaenv
 
-#' @importFrom tensorflow install_tensorflow
-#' @export
-tensorflow::install_tensorflow
-
 #' @importFrom tensorflow tensorboard
 #' @export
 tensorflow::tensorboard
 
-
-#' @importFrom tensorflow tf_config
+#' @importFrom tfruns flags
 #' @export
-tensorflow::tf_config
+tfruns::flags
+
+#' @importFrom tfruns flag_numeric
+#' @export
+tfruns::flag_numeric
+
+#' @importFrom tfruns flag_integer
+#' @export
+tfruns::flag_integer
+
+#' @importFrom tfruns flag_string
+#' @export
+tfruns::flag_string
+
+#' @importFrom tfruns flag_boolean
+#' @export
+tfruns::flag_boolean
+
+#' @importFrom tfruns run_dir
+#' @export
+tfruns::run_dir
+
+
+
 
 
 
