@@ -43,6 +43,23 @@ test_succeeds("model can be saved and loaded from yaml", {
   expect_equal(yaml, model_to_yaml(model_from))
 })
 
+test_succeeds("model can be saved and loaded from R 'raw' object", {
+  
+  if (!keras:::have_h5py())
+    skip("h5py not available for testing")
+  
+  model <- define_and_compile_model()
+  
+  mdl_raw <- serialize_model(model)
+  model <- unserialize_model(mdl_raw)
 
+})
+
+test_succeeds("saved models/weights are mirrored in the run_dir", {
+  run <- tfruns::training_run("train.R")
+  run_dir <- run$run_dir
+  expect_true(file.exists(file.path(run_dir, "model.h5")))
+  expect_true(file.exists(file.path(run_dir, "weights", "weights.h5")))
+})
 
 
