@@ -1,21 +1,9 @@
 
 skip_if_no_keras <- function(required_version = NULL) {
-  if (!have_keras(required_version))
+  if (!is_keras_available(required_version))
     skip("required keras version not available for testing")
 }
 
-have_keras <- function(required_version = NULL) {
-  implementation_module <- keras:::resolve_implementation_module()
-  if (reticulate::py_module_available(implementation_module)) {
-    if (!is.null(required_version))
-      keras:::keras_version() >= required_version
-    else
-      TRUE
-  } else {
-    FALSE
-  }
-}
-
 
 test_succeeds <- function(desc, expr, required_version = NULL) {
   test_that(desc, {
@@ -32,6 +20,11 @@ is_backend <- function(name) {
   identical(backend()$backend(), name)
 }
 
+skip_if_cntk <- function() {
+  if (is_backend("cntk"))
+    skip("Test not run for CNTK backend")
+}
+
 define_model <- function() {
   model <- keras_model_sequential() 
   model %>%
